/*
   Silly Language S1.

   Notice that these productions are mutually recursive, but no production is recursive.

   S1 -> A '+' B | B '+' A
   A  -> 'a' '*' '(' S1 ')' | 'a'
   B  -> 'b' '*' '(' S1 ')' | 'b'

   We can use EBNF to rewrite this grammar.

   S1 -> A '+' B | B '+' A
   A  -> 'a' [ '*' '(' S1 ')' ]
   B  -> 'b' [ '*' '(' S1 ')' ]

   Some example strings are:
   "a*(a+b)+b"
   "b*(a*(b+a)+b)+a*(a+b*(b+a))"


   This parser creates an abstract syntax tree for the expression.

   You can use this program to draw a picture of the AST.
   > java.exe ParseS1AST | dot.exe -Tpng -O

*/

public class ParseS1AST
{
   public static BTree parseS1(String stringInS1) throws Exception
   {
      Tokenizer tokens = new Tokenizer(stringInS1);  // Tokenizer is defined in Tokenizer.java

      return getS1(tokens);
   }//parecognizeS1()


   public static BTree getS1(Tokenizer tokens) throws Exception
   {
      BTree result = null;

      String token = tokens.peekToken();  // we "peek" ahead to help us
      if ( token.equals("a") )            // decide which case we're in
      {
         BTree temp = getA(tokens);
         token = tokens.nextToken();
         if ( !token.equals("+") )
         {
            throw new Exception("Can't parse " + token);
         }
         else
         {
            result = new BTree("+");
         }
         result.addLeftTree( temp );
         result.addRightTree( getB(tokens) );
      }
      else if ( token.equals("b") )
      {
         BTree temp = getB(tokens);
         token = tokens.nextToken();
         if ( !token.equals("+") )
         {
            throw new Exception("Can't parse " + token);
         }
         else
         {
            result = new BTree("+");
         }
         result.addLeftTree( temp );
         result.addRightTree( getA(tokens) );
      }
      else
         throw new Exception("Can't parse.");

      return result;
   }//getS1()


   public static BTree getA(Tokenizer tokens) throws Exception
   {
      BTree result = null;

      String token = tokens.nextToken();
      if ( !token.equals("a") )
      {
         throw new Exception("Can't parse " + token);
      }
      else
      {
         result = new BTree("a");
      }
      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // we "peek" ahead to help us
         if ( token.equals("*") )     // decide what to do next
         {
            tokens.nextToken();  // consume the *
            BTree temp = new BTree("*");
            temp.addLeftTree( result );
            result = temp;
            token = tokens.nextToken();
            if ( !token.equals("(") )
            {
               throw new Exception("Can't parse " + token);
            }
            result.addRightTree( getS1(tokens) );
            token = tokens.nextToken();
            if ( !token.equals(")") )
            {
               throw new Exception("Can't parse " + token);
            }
         }
      }

      return result;
   }//getA()


   public static BTree getB(Tokenizer tokens) throws Exception
   {
      BTree result = null;

      String token = tokens.nextToken();
      if ( !token.equals("b") )
      {
         throw new Exception("Can't parse " + token);
      }
      else
      {
         result = new BTree("b");
      }
      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // we "peek" ahead to help us
         if ( token.equals("*") )     // decide what to do next
         {
            tokens.nextToken();  // consume the *
            BTree temp = new BTree("*");
            temp.addLeftTree( result );
            result = temp;
            token = tokens.nextToken();
            if ( !token.equals("(") )
            {
               throw new Exception("Can't parse " + token);
            }
            result.addRightTree( getS1(tokens) );
            token = tokens.nextToken();
            if ( !token.equals(")") )
            {
               throw new Exception("Can't parse " + token);
            }
         }
      }

      return result;
   }//getB()





   public static void main(String[] args) throws Exception
   {
      String[] stringsInS1 = { "a * ( a + b ) + b",
                               "b * ( a * ( b + a ) + b ) + a * ( a + b * ( b + a ) )",
                               "b * ( a * ( b + a ) + b ) + a * ( b + b * ( b + a ) )" };

      for (int i = 0; i < stringsInS1.length - 1; i++)
      {
         BTree btree = parseS1( stringsInS1[i] );
         System.out.println( BTree2dot.btree2dot(btree) );
      }
   }//main()

}//ParseS1AST
