/*
   Silly Language S1.

   Notice that these productions are mutually recursive, but no production is recursive.

   S1 -> A '+' B | B '+' A
   A  -> 'a' '*' '(' S1 ')' | 'a'
   B  -> 'b' '*' '(' S1 ')' | 'b'

   We can use EBNF to rewrite this grammar.

   S1 -> A '+' B | B '+' A
   A  -> 'a' [ '*' '(' S1 ')' ]
   B  -> 'b' [ '*' '(' S1 ')' ]

   Some example strings are:
   "a*(a+b)+b"
   "b*(a*(b+a)+b)+a*(a+b*(b+a))"


   This parser creates a syntax tree for the expression.

   You can use this program to draw a picture of the syntax tree.
   > java.exe ParseS1ParseTree | dot.exe -Tpng -O

*/

public class ParseS1ParseTree
{
   public static Tree parseS1(String stringInS1) throws Exception
   {
      Tokenizer tokens = new Tokenizer(stringInS1);  // Tokenizer is defined in Tokenizer.java

      return getS1(tokens);
   }//parecognizeS1()


   public static Tree getS1(Tokenizer tokens) throws Exception
   {
      Tree result = new Tree( "S1" );

      String token = tokens.peekToken();  // we "peek" ahead to help us
      if ( token.equals("a") )            // decide which case we're in
      {
         result.addSubTree( getA(tokens) );
         token = tokens.nextToken();
         if ( !token.equals("+") )
         {
            throw new Exception("Can't parse " + token);
         }
         else
         {
            result.addSubTree( new Tree("+") );
         }
         result.addSubTree( getB(tokens) );
      }
      else if ( token.equals("b") )
      {
         result.addSubTree( getB(tokens) );
         token = tokens.nextToken();
         if ( !token.equals("+") )
         {
            throw new Exception("Can't parse " + token);
         }
         else
         {
            result.addSubTree( new Tree("+") );
         }
         result.addSubTree( getA(tokens) );
      }
      else
         throw new Exception("Can't parse.");

      return result;
   }//getS1()


   public static Tree getA(Tokenizer tokens) throws Exception
   {
      Tree result = new Tree( "A" );

      String token = tokens.nextToken();
      if ( !token.equals("a") )
      {
         throw new Exception("Can't parse " + token);
      }
      else
      {
         result.addSubTree( new Tree("a") );
      }
      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // we "peek" ahead to help us
         if ( token.equals("*") )     // decide what to do next
         {
            tokens.nextToken();  // consume the *
            result.addSubTree( new Tree("*") );
            token = tokens.nextToken();
            if ( !token.equals("(") )
            {
               throw new Exception("Can't parse " + token);
            }
            else
            {
               result.addSubTree( new Tree("(") );
            }
            result.addSubTree( getS1(tokens) );
            token = tokens.nextToken();
            if ( !token.equals(")") )
            {
               throw new Exception("Can't parse " + token);
            }
            else
            {
               result.addSubTree( new Tree(")") );
            }
         }
      }

      return result;
   }//getA()


   public static Tree getB(Tokenizer tokens) throws Exception
   {
      Tree result = new Tree( "B" );

      String token = tokens.nextToken();
      if ( !token.equals("b") )
      {
         throw new Exception("Can't parse " + token);
      }
      else
      {
         result.addSubTree( new Tree("b") );
      }
      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // we "peek" ahead to help us
         if ( token.equals("*") )     // decide what to do next
         {
            tokens.nextToken();  // consume the *
            result.addSubTree( new Tree("*") );
            token = tokens.nextToken();
            if ( !token.equals("(") )
            {
               throw new Exception("Can't parse " + token);
            }
            else
            {
               result.addSubTree( new Tree("(") );
            }
            result.addSubTree( getS1(tokens) );
            token = tokens.nextToken();
            if ( !token.equals(")") )
            {
               throw new Exception("Can't parse " + token);
            }
            else
            {
               result.addSubTree( new Tree(")") );
            }
         }
      }

      return result;
   }//getB()





   public static void main(String[] args) throws Exception
   {
      String[] stringsInS1 = { "a * ( a + b ) + b",
                               "b * ( a * ( b + a ) + b ) + a * ( a + b * ( b + a ) )",
                               "b * ( a * ( b + a ) + b ) + a * ( b + b * ( b + a ) )" };

      for (int i = 0; i < stringsInS1.length - 1; i++)
      {
         Tree tree = parseS1( stringsInS1[i] );
         System.out.println( Tree2dot.tree2dot(tree) );
      }
   }//main()

}//ParseS1ParseTree
