/*
   Silly Language S5.

   Notice that the first production is left recursive..

   S5 -> S5 '+' A | S5 '+' B | A | B
   A  -> 'a' '*' '(' S5 ')' | 'a'
   B  -> 'b' '*' '(' S5 ')' | 'b'

   We can use EBNF to rewrite this grammar.

   S5 -> [ S5 '+' ] [ A | B ]
   A  -> 'a' [ '*' '(' S5 ')' ]
   B  -> 'b' [ '*' '(' S5 ')' ]

   We can use EBNF to rewrite this grammar again, without the recursion.

   S5 -> [ A | B ] [ '+' [ A | B ] ]*
   A  -> 'a' [ '*' '(' S5 ')' ]
   B  -> 'b' [ '*' '(' S5 ')' ]

   We can simplify the grammar  bit.

   S5 -> C [ '+' C ]*
   C  -> A | B
   A  -> 'a' [ '*' '(' S5 ')' ]
   B  -> 'b' [ '*' '(' S5 ')' ]


   NOTE: This parser does not "do" anything. It does not create a parse tree
   nor an expression tree. This parser is just a "recognizer" for the language.
   It will throw an exception if the string cannot be parsed.
*/

public class ParseS5
{
   public static void recognizeS5(String stringInS5) throws Exception
   {
      Tokenizer tokens = new Tokenizer(stringInS5);  // Tokenizer is defined in Tokenizer.java

      getS5(tokens);
   }//parecognizeS5()


   public static void getS5(Tokenizer tokens) throws Exception
   {
      getC(tokens);
      while( tokens.hasToken() )
      {
         String token = tokens.peekToken();
         if ( token.equals("+") )
         {
            tokens.nextToken();  // consume the +
            getC(tokens);
         }
         else
            break;
      }
   }//getS5()


   public static void getC(Tokenizer tokens) throws Exception
   {
      String token = tokens.peekToken();
      if ( token.equals("a") )
      {
         getA(tokens);
      }
      else if ( token.equals("b") )
      {
         getB(tokens);
      }
      else
         throw new Exception("Can't parse.");
   }//getC()


   public static void getA(Tokenizer tokens) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("a") )
         throw new Exception("Can't parse " + token);
      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();
         if ( token.equals("*") )
         {
            tokens.nextToken();  // consume the *
            token = tokens.nextToken();
            if ( !token.equals("(") )
               throw new Exception("Can't parse " + token);
            getS5(tokens);
            token = tokens.nextToken();
            if ( !token.equals(")") )
               throw new Exception("Can't parse " + token);
         }
      }
   }//getA()


   public static void getB(Tokenizer tokens) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("b") )
         throw new Exception("Can't parse " + token);
      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();
         if ( token.equals("*") )
         {
            tokens.nextToken();  // consume the *
            token = tokens.nextToken();
            if ( !token.equals("(") )
               throw new Exception("Can't parse " + token);
            getS5(tokens);
            token = tokens.nextToken();
            if ( !token.equals(")") )
               throw new Exception("Can't parse " + token);
         }
      }
   }//getB()





   public static void main(String[] args) throws Exception
   {
      String[] stringsInS5 = { "a * ( a + b ) + a * ( a + b ) + b",
                               "b * ( a * ( b + a ) + b ) + a * ( a + b * ( b + a ) )",
                               "b * ( a * ( b + a ) + b ) + a * ( b + b * ( b + a ) )" };

      for (int i = 0; i < stringsInS5.length; i++)
      {
         System.out.println( "Parsing " + stringsInS5[i] );
         recognizeS5( stringsInS5[i] );
         System.out.println( " ... Parsed" );
      }
   }//main()

}//ParseS5
